//////////  Three and React Three Fiber Imports  //////////

import * as THREE from "three";
import { useRef } from "react";

import * as Types from "../../../types/world/worldTypes";
import Ground from "../../scene/environment/ground";
import { Bvh } from "@react-three/drei";
import AssetsGroupDuplicate from "./duplicateAsset/assetsGroupDuplicate";
import FloorGroupDuplicate from "./duplicateFloor/floorGroupDuplicate";
import WallsDuplicate from "./duplicateWall/wallsDuplicate";
import ZoneDuplicate from "./duplicateZone/zoneDuplicate";
import AislesDuplicate from "./duplicateAisle/aislesDuplicate";

export default function BuilderDuplicate({ projectId }: { projectId: string }) {
    const plane = useRef<THREE.Mesh>(null); // Reference for a plane object for raycaster reference.
    const grid = useRef() as any; // Reference for a grid object for raycaster reference.
    const lines = useRef([]) as Types.RefLines; // Reference for lines which stores all the lines that are ever drawn.
    const floorGroup = useRef() as Types.RefGroup; // Reference to the THREE.Group that has the roofs and the floors.

    //////////  Return  //////////

    return (
        <>
            <Ground grid={grid} plane={plane} />

            <WallsDuplicate
                lines={lines}
                projectId={projectId}
            />

            <Bvh firstHitOnly>

                <FloorGroupDuplicate
                    floorGroup={floorGroup}
                    lines={lines}
                    projectId={projectId}
                />

                <ZoneDuplicate projectId={projectId} />

                <AssetsGroupDuplicate projectId={projectId} />

                <AislesDuplicate projectId={projectId} />
            </Bvh>
        </>
    );
}
