import * as THREE from "three";
import { useEventsStore } from "../../../../store/simulation/useEventsStore";
import useModuleStore from "../../../../store/useModuleStore";
import { usePlayButtonStore } from "../../../../store/usePlayButtonStore";

function PointsDuplicate() {
    const { events } = useEventsStore();
    const { activeModule } = useModuleStore();
    const { isPlaying } = usePlayButtonStore();

    return (
        <>
            {activeModule === "simulation" && (
                <>
                    <group name="EventPointsGroup" visible={!isPlaying}>
                        {events.map((event, index) => {
                            const usedEvent = event;

                            if (usedEvent.type === "transfer") {
                                return (
                                    <group
                                        key={`${index}-${usedEvent.modelUuid}`}
                                        position={usedEvent.position}
                                        rotation={usedEvent.rotation}
                                    >
                                        {usedEvent.points.map((point, j) => (
                                            <mesh
                                                name="Event-Sphere"
                                                uuid={point.uuid}
                                                key={`${index}-${point.uuid}`}
                                                position={new THREE.Vector3(...point.position)}
                                                userData={{
                                                    modelUuid: usedEvent.modelUuid,
                                                    pointUuid: point.uuid,
                                                }}
                                            >
                                                <sphereGeometry args={[0.1, 16, 16]} />
                                                <meshStandardMaterial color="orange" />
                                            </mesh>
                                        ))}
                                    </group>
                                );
                            } else if (usedEvent.type === "vehicle") {
                                const point = usedEvent.point;
                                return (
                                    <group
                                        key={`${index}-${usedEvent.modelUuid}`}
                                        position={usedEvent.position}
                                        rotation={usedEvent.rotation}
                                    >
                                        <mesh
                                            name="Event-Sphere"
                                            uuid={point.uuid}
                                            position={new THREE.Vector3(...point.position)}
                                            userData={{
                                                modelUuid: usedEvent.modelUuid,
                                                pointUuid: point.uuid,
                                            }}
                                        >
                                            <sphereGeometry args={[0.1, 16, 16]} />
                                            <meshStandardMaterial color="blue" />
                                        </mesh>
                                    </group>
                                );
                            } else if (usedEvent.type === "roboticArm") {
                                const point = usedEvent.point;
                                return (
                                    <group
                                        key={`${index}-${usedEvent.modelUuid}`}
                                        position={usedEvent.position}
                                        rotation={usedEvent.rotation}
                                    >
                                        <mesh
                                            name="Event-Sphere"
                                            uuid={point.uuid}
                                            position={new THREE.Vector3(...point.position)}
                                            userData={{
                                                modelUuid: usedEvent.modelUuid,
                                                pointUuid: point.uuid,
                                            }}
                                        >
                                            <sphereGeometry args={[0.1, 16, 16]} />
                                            <meshStandardMaterial color="green" />
                                        </mesh>
                                    </group>
                                );
                            } else if (usedEvent.type === "machine") {
                                const point = usedEvent.point;
                                return (
                                    <group
                                        key={`${index}-${usedEvent.modelUuid}`}
                                        position={usedEvent.position}
                                        rotation={usedEvent.rotation}
                                    >
                                        <mesh
                                            name="Event-Sphere"
                                            uuid={point.uuid}
                                            position={new THREE.Vector3(...point.position)}
                                            userData={{
                                                modelUuid: usedEvent.modelUuid,
                                                pointUuid: point.uuid,
                                            }}
                                        >
                                            <sphereGeometry args={[0.1, 16, 16]} />
                                            <meshStandardMaterial color="purple" />
                                        </mesh>
                                    </group>
                                );
                            } else if (usedEvent.type === "storageUnit") {
                                const point = usedEvent.point;
                                return (
                                    <group
                                        key={`${index}-${usedEvent.modelUuid}`}
                                        position={usedEvent.position}
                                        rotation={usedEvent.rotation}
                                    >
                                        <mesh
                                            name="Event-Sphere"
                                            uuid={point.uuid}
                                            position={new THREE.Vector3(...point.position)}
                                            userData={{
                                                modelUuid: usedEvent.modelUuid,
                                                pointUuid: point.uuid,
                                            }}
                                        >
                                            <sphereGeometry args={[0.1, 16, 16]} />
                                            <meshStandardMaterial color="red" />
                                        </mesh>
                                    </group>
                                );
                            } else {
                                return null;
                            }
                        })}
                    </group>
                </>
            )}
        </>
    );
}

export default PointsDuplicate;
